#if !defined(__3dwindow_h)
#define __3dwindow_h

#include <slider.h>
#include <window.h>
#include <dc.h>
#include <groupbox.h>
#include "vector.h"
#include "color.h"

class CDiode;
class C3DObject;
class CTerminator;

//*** CLine ***
class CLine {
  int x1,y1,x2,y2;
  TColor color;
public:
  CLine() {x1=0;x2=0;y1=0;y2=0;};
  void Paint(TDC& dc);
  void SetLine(int ax1,int ay1,int ax2,int ay2,const TColor& acolor)
	 {x1=ax1; x2=ax2; y1=ay1; y2=ay2; color=acolor;};
};

//*** CPoint ***
class CPoint {
  int x,y;
  TColor color;
public:
  CPoint(const TColor& acolor) {x=0;y=0;color=acolor;};
  void Paint(TDC& dc);
  void SetPoint(int ax,int ay)
	 {x=ax; y=ay;};
  void SetColor(const TColor& acolor) {color=acolor;};
};

//*** C3DWindow ***
class C3DWindow : public TWindow {
public:
  int dx,dy;
  CMatrix matrix;
  CVector trans;
public:
  C3DWindow(TWindow* parent,int x, int y, int w, int h,
	 const CMatrix& amatrix, const CVector& atrans,const int adx =0,const int ady=0);
  void Rotate(int axe,double phi);
  void Line3D(const CVector& a,const CVector& b,const TColor& color, int xor=0);
  void Box3D(CVector& fix,CVector& x,CVector& y,CVector& z,const TColor& color);
  void Quadrat3D(const CVector& fix, const CVector& x, const CVector& y,const TColor& color);
  void Point3D(const CVector& fix,const TColor& color);
  void FatLine3D(const CVector& a, const CVector& b,const CVector& size, const TColor& color);
  void SetTransform(const CMatrix& amatrix,const CVector& atrans,const int adx=0,const int ady=0) {matrix=amatrix;trans=atrans;dx=adx;dy=ady;};
  void SetMatrix(const CMatrix& amatrix) {matrix=amatrix;};
  void SetShift(const int adx=0,const int ady=0) {dx=adx;dy=ady;};
  void SetTrans(const CVector& atrans) {trans=atrans;};
};

//*** CDiodeWindow ***
class CDiodeWindow: public C3DWindow {
  CDiode *diode;
  CPoint laser,realpos;
public:
  CDiodeWindow(CDiode *adiode, TWindow *parent,int x, int y, int w, int h,
	 const CMatrix& amatrix, const CVector& atrans,const int adx=0,const int ady=0);
  virtual void Paint(TDC& dc, BOOL erase, TRect& rect);
  void Update();
};

//*** CObject3DWindow ***
class CObject3DWindow: public C3DWindow {
public:
  C3DObject *object;
  int amplify;
public:
  CObject3DWindow(C3DObject *aobject, TWindow* parent,int x, int y, int w, int h);
  void SetAmplify(int aamplfiy);
  virtual void Paint(TDC& dc, BOOL erase, TRect& rect);
  void Update();
  void PaintNow();
};

//*** CTerminator3DWindow ***
class CTerminator3DWindow: public C3DWindow {
public:
  CTerminator *terminator;
  CLine line;
public:
  CTerminator3DWindow(CTerminator* aterminator,
	 TWindow* parent,int x, int y, int w, int h,
	 const CMatrix& matrix, const CVector& trans);
  virtual void Paint(TDC& dc, BOOL a, TRect& rec);
  virtual void Update();
  virtual void CalcTransform() {};
  void NewSize();
};

//*** CTerminatorYZWindow ***
class CTerminatorYZWindow: public CTerminator3DWindow {
public:
  CTerminatorYZWindow(CTerminator* aterminator, TWindow* parent,int ax, int ay, int aw, int ah);
  virtual void CalcTransform();
};

//*** CTerminatorXZWindow ***
class CTerminatorXZWindow: public CTerminator3DWindow {
public:
  CTerminatorXZWindow::CTerminatorXZWindow(CTerminator* aterminator, TWindow* parent,int ax, int ay, int aw, int ah);
  virtual void CalcTransform();
};

//*** CTerminatorYXWindow ***
class CTerminatorYXWindow: public CTerminator3DWindow {
  CPoint laser;
  CPoint electron;
public:
  CTerminatorYXWindow(CTerminator* aterminator, TWindow* parent,int x, int y, int w, int h);
  virtual void Paint(TDC& dc, BOOL a, TRect& rec);
  virtual void Update();
  virtual void CalcTransform();
};

#endif