#include "controls.h"
#include <groupbox.h>
#include <math.h>
#include <color.h>
#include <window.h>
#include <button.h>

static const TColor White(255,255,255);

//*** CXYControl ***
CXYControl::CXYControl(TWindow* aparent, int amyIDC, int x, int y,
  int xywidth,int xyheight, const char far* xtitle,const char far* ytitle,
  const char far* title,double alength,const char far* aunitangle,const char far* aunitdistance,
  int aprecisionangle,int aprecisiondistance,
  double afaktordistance, double afaktorangle,TModule* module)
			  :TWindow(aparent,title,module) {
  length=alength;
  faktorangle=afaktorangle;
  faktordistance=afaktordistance;
  precisionangle=aprecisionangle;
  precisiondistance=aprecisiondistance;
  unitangle=new char[strlen(aunitangle)];
  strcpy(unitangle,aunitangle);
  unitdistance=new char[strlen(aunitdistance)];
  strcpy(unitdistance,aunitdistance);
  parent=aparent;
  changed=0;
  myIDC=amyIDC;
  const dx=10;
  new TGroupBox(this,0,title,0,0,xywidth-1,xyheight-1);

  slidery=new TVSlider(this,IDC_XYSLIDERY,dx,25, 20,xyheight-45-2*dx);
  new TStatic(this,0,ytitle,20+2*dx,25,xywidth-3*dx-20,20);
  edity=new TEdit(this, IDC_XYEDITY,"",20+2*dx,20+3*dx,xywidth-4*dx-60,30, 20);
  staticx=new TStatic(this,0,unitangle,3*dx+20+xywidth-4*dx-60,25+3*dx,40,20);

  sliderx=new THSlider(this,IDC_XYSLIDERX,2*dx+20, xyheight-20-dx,xywidth-3*dx-20, 20);
  new TStatic(this,0,xtitle,2*dx+20,xyheight-65-3*dx,xywidth-3*dx-20,20);
  editx=new TEdit(this, IDC_XYEDITX,"",2*dx+20,xyheight-50-2*dx,xywidth-4*dx-60,30, 20);
  staticy=new TStatic(this,0,unitangle,3*dx+20+xywidth-4*dx-60,xyheight-45-2*dx,40,20);

  if (length!=0) {
	 aktunit=angle;
	 new TButton(this,IDC_CHANGEUNIT,"",xywidth-23,3,15,15);
  } else aktunit=nochange;

  Attr.X=x;
  Attr.Y=y;
  Attr.W=xywidth;
  Attr.H=xyheight;
  xmin=0;
  xmax=1;
  ymin=0;
  ymax=1;
  hxpos=0;
  hypos=0;
  xpos=0;
  ypos=0;

  BkBrush = new TBrush(::GetSysColor(COLOR_BTNFACE));
  WhiteBrush = new TBrush(White);
}

DEFINE_RESPONSE_TABLE1(CXYControl, TWindow)
  EV_WM_CTLCOLOR,
  EV_CHILD_NOTIFY_ALL_CODES(IDC_XYSLIDERX, SliderXChanged),
  EV_CHILD_NOTIFY_ALL_CODES(IDC_XYSLIDERY, SliderYChanged),
  EV_EN_KILLFOCUS(IDC_XYEDITX, ChangeXValue),
  EV_EN_KILLFOCUS(IDC_XYEDITY, ChangeYValue),
  EV_MESSAGE(IDC_RESETCHANGED, ResetChanged),
  EV_BN_CLICKED(IDC_CHANGEUNIT, ChangeUnit),
END_RESPONSE_TABLE;

void CXYControl::SetupWindow() {
  TWindow::SetupWindow();
  sliderx->SetRange(0,10000);
  slidery->SetRange(0,10000);
  sliderx->SetPosition(hxpos);
  slidery->SetPosition(hypos);
  slidery->SetRuler(20000);
  PrintDataY();
  PrintDataX();
}

void CXYControl::ChangeUnit() {
  if (aktunit==angle) {
	 aktunit=distance;
	 staticx->SetText(unitdistance);
	 staticy->SetText(unitdistance);
  } else {
	 aktunit=angle;
	 staticx->SetText(unitangle);
	 staticy->SetText(unitangle);
  }
  PrintDataX();
  PrintDataY();
}

void CXYControl::PrintDataX() {
  char buf[20];
  if (aktunit==distance) {
	 sprintf(buf,"%.*f",precisiondistance,length*sin(xpos)*faktordistance);
	 editx->SetText(buf);
  } else {
	 sprintf(buf,"%.*f",precisionangle,xpos*faktorangle);
	 editx->SetText(buf);
  }
}

void CXYControl::PrintDataY() {
  char buf[20];
  if (aktunit==distance) {
	 sprintf(buf,"%.*f",precisiondistance,length*sin(ypos)*faktordistance);
	 edity->SetText(buf);
  } else {
	 sprintf(buf,"%.*f",precisionangle,ypos*faktorangle);
	 edity->SetText(buf);
  }
}

HBRUSH CXYControl::EvCtlColor(HDC hDC, HWND hWndChild, UINT /*ctlType*/)
{
  if (hWndChild==editx->HWindow || hWndChild==edity->HWindow) {
	 TColor h(255,255,255);
	 ::SetBkColor(hDC,h);
	 return *WhiteBrush;
  } else {
	 ::SetBkColor(hDC,::GetSysColor(COLOR_BTNFACE));
	 return *BkBrush;
  }
}

void CXYControl::SetRange(double axmin,double axmax, double aymin, double aymax) {
  xmin=axmin;
  xmax=axmax;
  ymin=aymin;
  ymax=aymax;
}

void CXYControl::SetRuler(double ticGapX, double ticGapY, BOOL snapX, BOOL snapY) {
  sliderx->SetRuler(int((ticGapX)*10000/(xmax-xmin)),snapX);
  slidery->SetRuler(int((ticGapY)*10000/(xmax-xmin)),snapY);
}

void CXYControl::SetPosition(double axpos,double aypos, int slider) {
  xpos=axpos;
  ypos=aypos;
  hxpos=int((xpos-xmin)*10000/(xmax-xmin));
  hypos=int((ypos-ymin)*10000/(ymax-ymin));
  PrintDataY();
  PrintDataX();
  if (slider) {
	 sliderx->SetPosition(hxpos);
	 slidery->SetPosition(hypos);
  }
  changed=1;
}

void CXYControl::GetData(double& x,double& y,int reset) {
  x=xpos;
  y=ypos;
  if (reset) changed=0;
}

void CXYControl::SliderYChanged(UINT code) {
  ypos=(((double)(slidery->GetPosition())/10000*(ymax-ymin))+ymin);
  PrintDataY();
  changed=1;
  WPARAM sliderdown= (code==8) ? 1 : 0;
  parent->SendMessage(myIDC,sliderdown);
}

void CXYControl::SliderXChanged(UINT code) {
  xpos=(((double)(sliderx->GetPosition())/10000*(xmax-xmin))+xmin);
  PrintDataX();
  changed=1;
  WPARAM sliderdown= (code==8) ? 1 : 0;
  parent->SendMessage(myIDC,sliderdown);
}

void CXYControl::ChangeXValue() {
  char buf[64];
  double value;
  editx->GetText(buf,10);
  if (aktunit==distance) value=asin(atof(buf)/faktordistance/length);
  else value=atof(buf)/faktorangle;
  if ((value < xmin ) || (value > xmax)){
	 if (aktunit==distance) {
		sprintf(buf,"Value must be in the range %.*f/%.*f mm",precisiondistance,sin(xmin)*faktordistance*length,precisiondistance,sin(xmax)*faktordistance*length);
	 } else {
		sprintf(buf,"Value must be in the range %.*f/%.*f mrad",precisionangle,xmin*faktorangle,precisionangle,xmax*faktorangle);
	 }
	 MessageBox(buf);
  }
  else {
	 xpos=value;
	 sliderx->SetPosition(int((value-xmin)*10000/(xmax-xmin)));
	 changed=1;
	 parent->SendMessage(myIDC);
  }
  PrintDataX();
}

void CXYControl::ChangeYValue() {
  char buf[64];
  double value;
  edity->GetText(buf,10);
  if (aktunit==distance) value=asin(atof(buf)/faktordistance/length);
  else value=atof(buf)/faktorangle;
  if ((value < ymin ) || (value > ymax)){
	 if (aktunit==distance) {
		sprintf(buf,"Value must be in the range %.*f/%.*f mm",precisiondistance,sin(ymin)*faktordistance*length,precisiondistance,sin(ymax)*faktordistance*length);
	 } else {
		sprintf(buf,"Value must be in the range %.*f/%.*f mrad",precisionangle,ymin*faktorangle,precisionangle,ymax*faktorangle);
	 }
	 MessageBox(buf);
  }
  else {
	 ypos=value;
	 slidery->SetPosition(int((value-ymin)*10000/(ymax-ymin)));
	 changed=1;
	 parent->SendMessage(myIDC);
  }
  PrintDataY();
}

LRESULT CXYControl::ResetChanged(WPARAM,LPARAM) {
  changed=0;
  return 0;
};

//*** CXControl ***
const xfaktor=1;

DEFINE_RESPONSE_TABLE1(CXControl, TWindow)
  EV_WM_CTLCOLOR,
  EV_CHILD_NOTIFY_ALL_CODES(IDC_XSLIDER, SliderChanged),
  EV_EN_KILLFOCUS(IDC_XEDIT, ChangeValue),            // frappe dans le TEdit
END_RESPONSE_TABLE;

CXControl::CXControl(TWindow* aparent, int amyIDC, int x,int y,int xwidth,int xheight,
  const char far* titlex, const char far* title)
			  :TWindow(aparent,title,0) {
  parent=aparent;
  myIDC=amyIDC;
  const dx=5;
  new TGroupBox(this,0,title,0,0,xwidth-1,xheight-1);

  new TStatic(this,0,titlex,dx,dx+25,30,20, 10);
  edit=new TEdit(this, IDC_XEDIT,"",2*dx+30,dx+20,xwidth-4*dx-60,30, 20);
  new TStatic(this,0,"m",xwidth-dx-20,dx+25,20,20,10);
  slider=new THSlider(this,IDC_XSLIDER,dx,3*dx+40,xwidth-2*dx,20);

  changed=0;
  ok=0;
  Attr.X=x;
  Attr.Y=y;
  Attr.W=xwidth;
  Attr.H=xheight;
  min=0;
  max=1;
  hxpos=0;
  x=0;
  BkBrush = new TBrush(::GetSysColor(COLOR_BTNFACE));
  WhiteBrush = new TBrush(White);
}

void CXControl::SetupWindow() {
  TWindow::SetupWindow();
  slider->SetRange(0,10000);
  slider->SetPosition(hxpos);
  char text[20];
  sprintf(text,"%.3f",x*xfaktor);
  edit->SetText(text);
  ok=1;
}

HBRUSH CXControl::EvCtlColor(HDC hDC, HWND hWndChild, UINT /*ctlType*/)
{
  if (hWndChild==edit->HWindow) {
	 TColor h(255,255,255);
	 ::SetBkColor(hDC,h);
	 return *WhiteBrush;
  } else {
	 ::SetBkColor(hDC,::GetSysColor(COLOR_BTNFACE));
	 return *BkBrush;
  }
}

void CXControl::SetRange(double amin,double amax) {
  min=amin;
  max=amax;
}

/*void CXControl::SetRuler(double ticGap, BOOL snap) {
  slider->SetRuler(int((ticGap)*10000/(max-min)),snap);
} */

void CXControl::SetPosition(double axpos,int slidernew) {
  changed=1;
  x=axpos;
  hxpos=int((x-min)*10000/(max-min));
  if (slidernew) slider->SetPosition(hxpos);
  char text[20];
  sprintf(text,"%.3f",x*xfaktor);
  edit->SetText(text);
}

double CXControl::GetData(int reset) {
  if (reset) changed=0;
  return x;
}

void CXControl::SliderChanged(UINT) {
  char text[20];
  x=(((double)(slider->GetPosition())/10000*(max-min))+min)*xfaktor;
  sprintf(text,"%.3f",x);
  edit->SetText(text);
  changed=1;
  parent->SendMessage(myIDC);
}

void CXControl::ChangeValue() {
  char buf[64];
  double value;
  edit->GetText(buf,10);
  value=atof(buf)/xfaktor;
  if ((value < min ) || (value > max)){
	 sprintf(buf,"Value must be in the range %.3f/%.3f !",min*xfaktor,max*xfaktor);
	 MessageBox(buf);
  }
  else {
	 x=value;
	 slider->SetPosition(int((value-min)*10000/(max-min)));
	 changed=1;
	 parent->SendMessage(myIDC);
  }
  sprintf(buf,"%.3f",x);
  edit->SetText(buf);
}

//*** CSizeControl ***
const sizefaktor=1;
const sizeheight=90;
const sizewidth=175;

DEFINE_RESPONSE_TABLE1(CSizeControl, TWindow)
  EV_WM_CTLCOLOR,
  EV_CHILD_NOTIFY_ALL_CODES(IDC_SIZESLIDER, SliderChanged),
  EV_EN_KILLFOCUS(IDC_SIZEEDIT, ChangeValue),            // frappe dans le TEdit
END_RESPONSE_TABLE;

CSizeControl::CSizeControl(TWindow* aparent, int amyIDC, int x,int y,
  const char far* title, TModule* module)
			  :TWindow(aparent,title,module) {
  parent=aparent;
  myIDC=amyIDC;
  const dx=10;
  new TGroupBox(this,0,title,0,0,sizewidth-1,sizeheight-1);

  new TStatic(this,0,"Size =",dx,dx+25,30,20, 10);
  edit=new TEdit(this, IDC_SIZEEDIT,"",2*dx+30,dx+20,sizewidth-4*dx-60,30, 20);
  new TStatic(this,0,"m",sizewidth-dx-20,dx+25,20,20,10);
  slider=new THSlider(this,IDC_SIZESLIDER,dx,3*dx+40,sizewidth-2*dx,20);

  changed=0;
  ok=0;
  Attr.X=x;
  Attr.Y=y;
  Attr.W=sizewidth;
  Attr.H=sizeheight;
  min=0;
  max=1;
  hsizepos=0;
  size=0;

  BkBrush = new TBrush(::GetSysColor(COLOR_BTNFACE));
  WhiteBrush = new TBrush(White);
}

void CSizeControl::SetupWindow() {
  TWindow::SetupWindow();
  slider->SetRange(0,10000);
  slider->SetPosition(hsizepos);
  double sizepos=((double)(slider->GetPosition())/10000*(max-min))+min;
  char text[20];
  sprintf(text,"%.1e",pow(10,sizepos*sizefaktor));
  edit->SetText(text);
  ok=1;
}

HBRUSH CSizeControl::EvCtlColor(HDC hDC, HWND hWndChild, UINT /*ctlType*/)
{
  if (hWndChild==edit->HWindow) {
	 TColor h(255,255,255);
	 ::SetBkColor(hDC,h);
	 return *WhiteBrush;
  } else {
	 ::SetBkColor(hDC,::GetSysColor(COLOR_BTNFACE));
	 return *BkBrush;
  }
}

void CSizeControl::SetRange(double amin,double amax) {
  min=log10(amin);
  max=log10(amax);
}

/*void CSizeControl::SetRuler(double ticGap, BOOL snap) {
  slider->SetRuler(int((ticGap)*10000/(max-min)),snap);
} */

void CSizeControl::SetPosition(double sizepos) {
  changed=1;
  size=sizepos;
  sizepos=log10(sizepos);
  hsizepos=int((sizepos-min)*10000/(max-min));
  slider->SetPosition(hsizepos);
  char text[20];
  sprintf(text,"%.2e",pow(10,sizepos*sizefaktor));
  edit->SetText(text);
}

double CSizeControl::GetData(int reset) {
  if (reset) changed=0;
  return size;
}

void CSizeControl::SliderChanged(UINT) {
  char text[20];
  size=pow(10,(((double)(slider->GetPosition())/10000*(max-min))+min)*sizefaktor);
  sprintf(text,"%.2e",size);
  edit->SetText(text);
  changed=1;
  parent->SendMessage(myIDC);
}

void CSizeControl::ChangeValue() {
  char buf[64];
  double value;
  edit->GetText(buf,10);
  value=log10(atof(buf)/sizefaktor);
  if ((value < min ) || (value > max)){
	 sprintf(buf,"Value must be in the range %.2e/%.2e !",pow(10,min*sizefaktor),pow(10,max*sizefaktor));
	 MessageBox(buf);
  }
  else {
	 size=value;
	 slider->SetPosition(int((value-min)*10000/(max-min)));
	 changed=1;
	 parent->SendMessage(myIDC);
  }
  sprintf(buf,"%.2e",size);
  edit->SetText(buf);
}

//*** CSLControl ***
const slfaktor=1;
const slheight=180;
const slwidth=180;

DEFINE_RESPONSE_TABLE1(CSLControl, TWindow)
  EV_WM_CTLCOLOR,
  EV_CHILD_NOTIFY_ALL_CODES(IDC_SIZESLIDER, SliderSChanged),
  EV_CHILD_NOTIFY_ALL_CODES(IDC_LONGSLIDER, SliderLChanged),
  EV_EN_KILLFOCUS(IDC_SIZEEDIT, ChangeSValue),
  EV_EN_KILLFOCUS(IDC_LONGEDIT, ChangeLValue),
END_RESPONSE_TABLE;

CSLControl::CSLControl(TWindow* aparent, int amyIDC, int x,int y,
  const char far* stitle, const char far* ltitle, TModule* module)
			  :TWindow(aparent,0,module) {
  parent=aparent;
  myIDC=amyIDC;
  const dx=10;
  const dy=8;
  new TGroupBox(this,0,0,0,0,slwidth-1,slheight-1);

  new TStatic(this,0,stitle,dx,2,150,20);
  new TStatic(this,0,"X =",dx,dy+25,30,20, 10);
  edits=new TEdit(this, IDC_SIZEEDIT,"",2*dx+30,dy+20,slwidth-4*dx-60,30, 20);
  new TStatic(this,0,"m",slwidth-dx-20,dy+25,20,20,10);
  sliders=new THSlider(this,IDC_SIZESLIDER,dx,3*dy+40,slwidth-2*dx,20);
  const h=4*dy+55;
  new TStatic(this,0,ltitle,dx,h,150,20);
  new TStatic(this,0,"Y =",dx,dy+25+h,30,20, 10);
  editl=new TEdit(this, IDC_LONGEDIT,"",2*dx+30,dy+20+h,slwidth-4*dx-60,30, 20);
  new TStatic(this,0,"m",slwidth-dx-20,dy+25+h,20,20,10);
  sliderl=new THSlider(this,IDC_LONGSLIDER,dx,3*dy+40+h,slwidth-2*dx,20);

  ok=0;
  Attr.X=x;
  Attr.Y=y;
  Attr.W=slwidth;
  Attr.H=slheight;
  minl=0;
  maxl=1;
  mins=0;
  maxs=1;
  hsizepos=0;
  hlongpos=0;
  WhiteBrush = new TBrush(White);
  BkBrush = new TBrush(::GetSysColor(COLOR_BTNFACE));
}

void CSLControl::SetupWindow() {
  TWindow::SetupWindow();
  sliderl->SetRange(0,10000);
  sliderl->SetPosition(hlongpos);
  double longpos=((double)(sliderl->GetPosition())/10000*(maxl-minl))+minl;
  char text[20];
  sprintf(text,"%.3f",longpos*slfaktor);
  editl->SetText(text);

  sliders->SetRange(0,10000);
  sliders->SetPosition(hsizepos);
  double sizepos=((double)(sliders->GetPosition())/10000*(maxs-mins))+mins;
  sprintf(text,"%.2e",pow(10,sizepos*slfaktor));
  edits->SetText(text);
  ok=1;
}

HBRUSH CSLControl::EvCtlColor(HDC hDC, HWND hWndChild, UINT /*ctlType*/)
{
  if (hWndChild==edits->HWindow || hWndChild==editl->HWindow) {
	 TColor h(255,255,255);
	 ::SetBkColor(hDC,h);
	 return *WhiteBrush;
  } else {
	 ::SetBkColor(hDC,::GetSysColor(COLOR_BTNFACE));
	 return *BkBrush;
  }
}

void CSLControl::SetRange(double amins,double amaxs,double aminl,double amaxl) {
  mins=log10(amins);
  maxs=log10(amaxs);
  minl=aminl;
  maxl=amaxl;
}

/*void CSLControl::SetRuler(double ticGap, BOOL snap) {
  slider->SetRuler(int((ticGap)*10000/(max-min)),snap);
} */

void CSLControl::SetPosition(double sizepos,double longpos) {
  sizepos=log10(sizepos);
  hsizepos=int((sizepos-mins)*10000/(maxs-mins));
  sliders->SetPosition(hsizepos);
  char text[20];
  sprintf(text,"%.2e",pow(10,sizepos*slfaktor));
  edits->SetText(text);

  longpos=longpos;
  hlongpos=int((longpos-minl)*10000/(maxl-minl));
  sliderl->SetPosition(hlongpos);
  sprintf(text,"%.3f",longpos*slfaktor);
  editl->SetText(text);
}

void CSLControl::GetData(double &asize, double &along) {
  asize= (ok) ? pow(10,(((double)(sliders->GetPosition())/10000*(maxs-mins))+mins)*slfaktor)
				  : pow(10,(((double)(hsizepos)/10000*(maxs-mins))+mins)*slfaktor);
  along= (ok) ? (((double)(sliderl->GetPosition())/10000*(maxl-minl))+minl)*slfaktor
				  : (((double)(hlongpos)/10000*(maxl-minl))+minl)*slfaktor;

}

void CSLControl::SliderSChanged(UINT code) {
  char text[20];
  sprintf(text,"%.2e",pow(10,(((double)(sliders->GetPosition())/10000*(maxs-mins))+mins)*slfaktor));
  edits->SetText(text);
  WPARAM sliderdown= (code==8) ? 1 : 0;
  parent->SendMessage(myIDC,sliderdown);
}

void CSLControl::SliderLChanged(UINT code) {
  char text[20];
  sprintf(text,"%.3f",(((double)(sliderl->GetPosition())/10000*(maxl-minl))+minl)*slfaktor);
  editl->SetText(text);
  WPARAM sliderdown= (code==8) ? 1 : 0;
  parent->SendMessage(myIDC,sliderdown);
}

void CSLControl::ChangeSValue() {
  char buf[64];
  double value;
  edits->GetText(buf,10);
  value=log10(atof(buf)/slfaktor);
  if ((value < mins ) || (value > maxs)){
	 sprintf(buf,"Value must be in the range %.2e/%.2e !",pow(10,mins*slfaktor),pow(10,maxs*slfaktor));
	 MessageBox(buf);
  } else {
	 sliders->SetPosition(int((value-mins)*10000/(maxs-mins)));
	 parent->SendMessage(myIDC);
  }
  sprintf(buf,"%.2e",(((double)(sliders->GetPosition())/10000*(maxs-mins))+mins)*slfaktor);
  edits->SetText(buf);
}

void CSLControl::ChangeLValue() {
  char buf[64];
  double value;
  editl->GetText(buf,10);
  value=(atof(buf)/slfaktor);
  if ((value < minl ) || (value > maxl)){
	 sprintf(buf,"Value must be in the range %.3f/%.3f !",minl*slfaktor,maxl*slfaktor);
	 MessageBox(buf);
  } else {
	 sliderl->SetPosition(int((value-minl)*10000/(maxl-minl)));
	 parent->SendMessage(myIDC);
  }
  sprintf(buf,"%.3f",(((double)(sliderl->GetPosition())/10000*(maxl-minl))+minl)*slfaktor);
  editl->SetText(buf);
}


